////////////////////////////////////////
// Function: buttonDebounce - Test that no buttons are pressed for at least cont_time milliseconds 
// Parameters:
//   t - number of milliseconds of continuous no buttons pressed
// Returns:
//   None
////////////////////////////////////////
void buttonDebounce(int t)
{
   int count = t;

   while ( count > 0 )
   {
      count --;
      if ((analogRead(BUTTONS_APIN) <= 962 ) || (digitalRead(BUTTON_ACTIVATE_PIN) == LOW))
      {
         count = t;
      }
      delay(1);
   }
}

////////////////////////////////////////
// Function: detectButtonPress - Figures out which button was pressed and does ss debounce
// Parameters:
//   waitForRelease - Wait until the button is released and do debounce
// Returns:
//   The button that was pressed
//
// DETAILED DESCRIPTION BELOW
//
// To allow multiple buttons to work off a single pin the Camera Axe attaches
// 6 buttons with some resistors in this configuration.
//           1K       1K        1K        1K        1K        1K
//  5V____/\/\/\____/\/\/\____/\/\/\____/\/\/\____/\/\/\____/\/\/\____/\/\/\____/\/\/\____GND
//                |         |         |         |         |         |         |         |
//                |         |         |         |         |         |         |         |
//                  /         /         /         /         /         /         /         /
//                 /         /         /         /         /         /         /         /
//                |         |         |         |         |         |         |         |
//                |         |         |         |         |         |         |         |
// BUTTONS_PIN---------------------------------------------------------------------------
//
// During init we set the button pin using a 20K internal pullup resistor
// in the microcontroller so ths signal isn't floating when no buttons are
// pressed.  Using some simple parallel/serial resistor and voltage
// divider equations I get get these ideal values:
//
// EX (first Equation): (((1K*20K)/(1K+20K))/(((1K*20K)/(1K+20K)+7000)))*5
//
// No button pressed:            5.0000 V  =  1023  [range          >941]
// Menu forward button pressed:  4.2000 V  =  859   [range <=941 && >781]
// Menu backward button pressed: 3.4375 V  =  703   [range <=781 && >625]
// Up button pressed:            2.6744 V  =  547   [range <=625 && >465]
// Down button pressed:          1.8750 V  =  384   [range <=465 && >294]
// Left button pressed:          1.0000 V  =  205   [range <=294 && >102]
// Right button pressed:         0.0000 V  =  0     [range <=102        ]
//
////////////////////////////////////////
byte detectButtonPress(boolean waitForRelease)
{
  int buttonVal[3];
  byte button[3];
  byte i;

  buttonVal[0] = analogRead(BUTTONS_APIN);
  buttonVal[1] = analogRead(BUTTONS_APIN);
  buttonVal[2] = analogRead(BUTTONS_APIN);

  for(i=0; i<3; ++i)
  {
    if (buttonVal[i] > 941)
      button[i] = BUTTON_NONE;
    else if (buttonVal[i] <= 941 && buttonVal[i] > 781)
      button[i] = BUTTON_MENU_F;
    else if (buttonVal[i] <= 781 && buttonVal[i] > 625)
      button[i] = BUTTON_MENU_B;
    else if (buttonVal[i] <= 625 && buttonVal[i] > 465)
      button[i] = BUTTON_UP;
    else if (buttonVal[i] <= 465 && buttonVal[i] > 294)
      button[i] = BUTTON_DOWN;
    else if (buttonVal[i] <= 294 && buttonVal[i] > 102)
      button[i] = BUTTON_LEFT;
    else //(buttonVal <= 102)
      button[i] = BUTTON_RIGHT;
  }
  
  // If 3 button readings don't match just go with none (happens during debounce)
  if ((button[0] != button[1]) || (button[0] != button[2]))
    return BUTTON_NONE;

  if (digitalRead(BUTTON_ACTIVATE_PIN) == LOW)
    button[0] = BUTTON_ACTIVATE;

  if ((button[0] != BUTTON_NONE) && waitForRelease)
  {
    // Spin waiting for the button to be released
    while ( (analogRead(BUTTONS_APIN) <= 941 ) || (digitalRead(BUTTON_ACTIVATE_PIN) == LOW) )
    {
      delay(1);
    }

    buttonDebounce(10);  // 10 ms debounce
  }
  return button[0];
}


