
// Drives the NHD-C0216CiZ-FSW-FBW-3V3 LCD

byte g_i2cAddr;
byte g_cmdDelay = 1;

const byte LCD_DISP_CMD       = 0x0;        // Command for display
const byte LCD_RAM_WRITE_CMD  = 0x40;       // Write to display ram
const byte LCD_CLEAR_DISP_CMD = 0x01;       // Clear display
const byte LCD_DISP_ON_CMD    = 0x0C;       // Display on
const byte LCD_DISP_OFF_CMD   = 0x08;       // Display off
const byte LCD_SET_DDRAM_CMD  = 0x80;       // Set ddram address
const byte LCD_CONTRAST_CMD   = 0x70;       // Set lcd contrast
const byte LCD_CURSOR_ON_BIT  = (1 << 1);   // Cursor selection bit
const byte LCD_BLINK_ON_BIT   = (1 << 0);   // Blink selection bit

void lcdInit(byte numLines, byte numCols, byte i2cAddr)
{
  byte status;
  
  g_i2cAddr  = i2cAddr >> 1; // Arduino requires address shifted like this
  
  // Below code is straight from data sheet for NHD-C0216CiZ-FSW-FBW-3V3
  delay(10);
  Wire.begin();
  Wire.beginTransmission(g_i2cAddr);  
  Wire.write((byte)0x0);
  Wire.write(0x38);
  delay(10);
  Wire.write(0x39);
  delay(10);
  Wire.write(0x14);
  Wire.write(0x78);
  Wire.write(0x5E);
  Wire.write(0x6D);
  Wire.write(0x0C);
  Wire.write(0x01);
  Wire.write(0x06);
  delay(10);
  status = Wire.endTransmission();
  delay(g_cmdDelay);
  //Serial.println(status);
}

void lcdCommand(byte val)
{
  Wire.beginTransmission(g_i2cAddr);
  Wire.write(LCD_DISP_CMD);
  Wire.write(val);
  Wire.endTransmission();
  delay(g_cmdDelay);
}

void lcdPrintDigit(byte val)
{
  Wire.beginTransmission(g_i2cAddr);
  Wire.write(LCD_RAM_WRITE_CMD);
  Wire.write(val+48); // +48 converts to ascii
  Wire.endTransmission();
}

void lcdPrintChar(byte val)
{
  Wire.beginTransmission(g_i2cAddr);
  Wire.write(LCD_RAM_WRITE_CMD);
  Wire.write(val);
  Wire.endTransmission();
}

void lcdPrintStr(const char *str)
{
  byte length = strlen(str);
  
  Wire.beginTransmission(g_i2cAddr);
  Wire.write ( LCD_RAM_WRITE_CMD );
  Wire.write ( (uint8_t *)str, length );
  Wire.endTransmission ();
}    

void lcdPrintStr(byte x, byte y, const char *str)
{
  byte length = strlen(str);

  lcdSetCursorPos(x, y);

  Wire.beginTransmission(g_i2cAddr);
  Wire.write ( LCD_RAM_WRITE_CMD );
  Wire.write ( (uint8_t *)str, length );
  Wire.endTransmission ();
}

void lcdPrintStrList(const char *str)
{
  byte i = 0;

  // Search for null terminated string or use the special '|' char to terminate a string
  // The '|' termination makes it easier to do lists of strings in PROGMEM
  do {
    lcdPrintChar(str[i]);
    ++i;
  }while (str[i] != '\0' && str[i] != '|');
}

void lcdClear()
{
  lcdCommand (LCD_CLEAR_DISP_CMD);
}

// val 0=off, 1=on
void lcdOn(byte val)
{
  if (val == 0)
    lcdCommand(LCD_DISP_OFF_CMD);
  else
    lcdCommand(LCD_DISP_ON_CMD);
}

void lcdSetShowCursor(byte val)
{
  if (val == 0)
    lcdCommand(LCD_DISP_ON_CMD & ~LCD_CURSOR_ON_BIT);
  else
    lcdCommand(LCD_DISP_ON_CMD | LCD_CURSOR_ON_BIT);
}

void lcdSetBlinkCursor(byte val)
{
  if (val == 0)
    lcdCommand(LCD_DISP_ON_CMD & ~LCD_BLINK_ON_BIT);
  else
    lcdCommand(LCD_DISP_ON_CMD | LCD_BLINK_ON_BIT);
}

void lcdSetCursorPos(byte x, byte y)
{
  if (y == 1)
    y = 0x40;
  
  lcdCommand(LCD_SET_DDRAM_CMD + y + x);
}

void lcdSetBacklight(byte pin, byte val)
{
  analogWrite( pin, val);
}

// Valid range is 0->7 (0 is light and 7 is dark)
void lcdSetContrast(byte val)
{
  lcdCommand(LCD_CONTRAST_CMD + val);
}

////////////////////////////////////////
// Function: lcdPrintZeros - Prints a number to the LCD and fills in empty digits with zeros
// Parameters:
//   x            - X position on LCD to start drawing (top left is 0,0)
//   y            - Y position on LCD to start drawing (top left is 0,0)
//   bcdVal       - The value being displayed, it must be passed in as a BCD (binary coded decimal)
//   numDigits    - Number of digits being printed (4 is the max) (ex:: bcdVal-45 and numDigit-3 would print 045)
//   numDigitsDec  - Number of digits in this number before the decimal (total digits max is 4 not counting a decimal) (ex:: bcdVal-45 and numDigit-3 would print 045)
//   numDigitsFrac - Number of digits in this number after decimal (if this is 0 then the decimal is not displayed)
// Returns:
//   None
////////////////////////////////////////
void lcdPrintZeros(byte x, byte y, unsigned int bcdVal, byte numDigitsDec, byte numDigitsFrac)
{
  byte i;
  unsigned int tempBcd;
  byte numDigits = numDigitsDec + numDigitsFrac;

  lcdSetCursorPos(x,y);

  for(i=0; i<numDigits; ++i)
  {
    tempBcd = bcdVal >> (numDigits-1-i)*4;
    lcdPrintDigit(tempBcd & 0xF);
    
    if (numDigitsFrac && (i == (numDigitsDec-1)))
      lcdPrintChar('.');
  }
}

////////////////////////////////////////
// Function: lcdSetNumber - Sets a number using buttons and the LCD (up to increase number, down decrease number, left
//     to move to left digit, right to move to right digit) (numbers wrap from 9->0 and 0->9)
// Parameters:
//   x            - X position on LCD to start drawing (top left is 0,0)
//   y            - Y position on LCD to start drawing (top left is 0,0)
//   selected     - The current value should be selected (ie xor box drawn around it)
//   cursorPos    - Position of the cursor (0 is all the way to the right) - (modified value is returned by reference)
//   eepromPos    - Location in the list of values stored in the eeprom
//   maxDigit     - Max most significant digit (useful for time where you only want to allow 59 seconds)
//   numCharsDec  - Number of chars in this number before the decimal
//   numCharsFrac - Number of chars in this number after decimail (if this is 0 then the decimal is not displayed)
// Returns:
//   Last button pressed which caused us to exit this function
////////////////////////////////////////
byte lcdSetNumber(byte x, byte y, boolean selected, byte *cursorPos, byte eepromPos, byte maxDigit, byte numCharsDec, byte numCharsFrac)
{
  int val = eepromReadInt(eepromPos);
  unsigned int bcdVal = decimalToBcd(val);
  byte button;
  byte adjNumChars = (numCharsFrac) ? (numCharsDec+numCharsFrac+1) : numCharsDec;
  const unsigned int bcdClearMask[4] = {0xFFF0, 0xFF0F, 0xF0FF, 0x0FFF};
  
  if (!selected)
  {
    lcdPrintZeros(x, y, bcdVal, numCharsDec, numCharsFrac);
    return BUTTON_NONE;
  }

  *cursorPos = min(*cursorPos, adjNumChars-1); // Clamp cursor position

  button = detectButtonPress(true);

  //if (edit)
  {
    switch (button)
    {
      case BUTTON_NONE:
      case BUTTON_MENU_F:
      case BUTTON_MENU_B:
      case BUTTON_ACTIVATE:
          break;  // Do nothing
      case BUTTON_UP:
      case BUTTON_DOWN:
        {
          unsigned int tempBcd = bcdVal;
          byte adjCursorPos;  
          
          if (numCharsFrac && (*cursorPos > numCharsFrac))
            adjCursorPos = (*cursorPos-1);
          else
            adjCursorPos = *cursorPos;

          tempBcd = (tempBcd >> (adjCursorPos*4)) & 0xF;  // Isolate single digit in BCD number that is changing

          if (button == BUTTON_UP)
          {
            if ((tempBcd == maxDigit) && (adjCursorPos == adjNumChars-1))
              tempBcd = 0;
            else if (tempBcd == 9)
              tempBcd = 0;
            else
              ++tempBcd;
          }
          else // BUTTON_DOWN
          {
            if ((tempBcd == 0) && (adjCursorPos == adjNumChars-1))
              tempBcd = maxDigit;
            else if (tempBcd == 0)
              tempBcd = 9;
            else
              --tempBcd;
          }

          bcdVal = (bcdVal & bcdClearMask[adjCursorPos]) | (tempBcd << (adjCursorPos*4));  // Mask out BCD character changing and replace with new
          val = bcdToDecimal(bcdVal);
          eepromWriteInt(eepromPos, val);
       }
        break;
      case BUTTON_LEFT:
        {
          *cursorPos = (*cursorPos + 1) % adjNumChars;
          if (numCharsFrac && (*cursorPos == numCharsFrac))  // special - adjusts for decimal point
            ++(*cursorPos);
        }
        break;
      case BUTTON_RIGHT:
        {
          if (*cursorPos == 0)
            *cursorPos = adjNumChars-1;
          else
            --(*cursorPos);

          if (numCharsFrac && (*cursorPos == numCharsFrac))  // special - adjusts for decimal point
            --(*cursorPos);
        }
        break;
    }
  }

  lcdPrintZeros(x, y, bcdVal, numCharsDec, numCharsFrac);
  
  // Show curser on digit being edited
  g_showCursorX = x+adjNumChars-1-(*cursorPos);
  g_showCursorY = y;
  return button;
}

////////////////////////////////////////
// Function: lcdSetNumber - Sets a number using buttons and the LCD (up to increase number, down decrease number, left
//     to move to left digit, right to move to right digit) (numbers wrap from 9->0 and 0->9)
// Parameters:
//   x                 - X position on LCD to start drawing (top left is 0,0)
//   y                 - Y position on LCD to start drawing (top left is 0,0)
//   selected          - The current value should be selected (ie xor box drawn around it)
//   cursorPos         - Position of the cursor (0 is all the way to the right) - (modified value is returned by reference)
//   bankPos           - Position of the selected value out of the 3 different banks (0 is all the way to the right)
//   eepromPosSec      - Location in the list of values stored in the eeprom
//   eepromPosMilliSec - Location in the list of values stored in the eeprom
//   eepromPosMicroSec - Location in the list of values stored in the eeprom

// Returns:
//   Last button pressed which caused us to exit this function
////////////////////////////////////////
byte lcdSetNumber3(byte x, byte y, boolean selected, byte *cursorPos, byte *bankPos, byte eepromPosSec, byte eepromPosMilliSec, byte eepromPosMicroSec)
{
  byte button = BUTTON_NONE;
  boolean active[3] = {false, false, false};

  active[*bankPos] = selected;

  button |= lcdSetNumber(x, y, active[0], cursorPos, eepromPosSec, 9, 3, 0);
  lcdSetCursorPos(x+3, y);
  lcdPrintChar(':');
  button |= lcdSetNumber(x+4, y, active[1], cursorPos, eepromPosMilliSec, 9, 3, 0);
  lcdSetCursorPos(x+7, y);
  lcdPrintChar(':');
  button |= lcdSetNumber(x+8, y, active[2], cursorPos, eepromPosMicroSec, 9, 3, 0);
  
  if (*cursorPos == 2 && button == BUTTON_RIGHT)
    *bankPos = (*bankPos+1)%3;
  if (*cursorPos == 0 && button == BUTTON_LEFT)
    *bankPos = (*bankPos!=0) ? *bankPos-1 : 2;
    
  return button;
}

////////////////////////////////////////
// Function: lcdSetString - Sets a string using buttons and the LCD
// Parameters:
//   x            - X position on LCD to start drawing (top left is 0,0)
//   y            - Y position on LCD to start drawing (top left is 0,0)
//   selected     - The current value should be selected (ie xor box drawn around it)
//   numStrings   - The number of strings in the list of string ("strings")
//   stringLength - The lenght of a string (all strings should be the same length)
//   eepromPos    - Location in the list of values stored in the eeprom
//   strings      - A list of strings that the user can select from
// Returns:
//   Last button pressed which caused us to exit this function
////////////////////////////////////////
byte lcdSetString(byte x, byte y, boolean selected, byte numStrings, byte stringLength, byte eepromPos, const char *strings)
{
  byte button;
  byte val = eepromReadInt(eepromPos);
  
  if (val > numStrings)
  {
     // For the ditial/analog threshold case we can run into values too big
     val = numStrings-1;
  }
  
  if (!selected)
  {
    // If this value is not selected nothing can change so just print out the value
    lcdSetCursorPos(x, y);
    lcdPrintStr(strings+(stringLength+1)*val);
    return BUTTON_NONE;
  }
  
  button = detectButtonPress(true);
  
  switch (button)
  {
    case BUTTON_NONE:
    case BUTTON_MENU_F:
    case BUTTON_MENU_B:
    case BUTTON_LEFT:
    case BUTTON_RIGHT:
    case BUTTON_ACTIVATE:
        break;  // Do nothing
    case BUTTON_UP:
      {
        if (val == 0)
          val = numStrings - 1;
        else
          --val;
        eepromWriteInt(eepromPos, val);
      }
      break;
    case BUTTON_DOWN:
      {
        val = (val+1)%numStrings;
        eepromWriteInt(eepromPos, val);
      }
      break;
  }     

  lcdSetCursorPos(x, y);
  lcdPrintStrList(strings+(stringLength+1)*val);
  
  // Show curser on digit being edited
  g_showCursorX = x;
  g_showCursorY = y;
  
  return button;
}

